// Parser.java
//
// Informatics 102 Spring 2012
// Code Example
//
// This class implements a recursive-descent parser for the example grammar
// given in the README included with this code example.  For convenience,
// the rule corresponding to each of the parseXXX() methods is listed above
// the code for that rule.
//
// This parser makes use of the built-in Scanner class in Java.  For many
// recusive-descent parsers, this class won't be quite good enough -- because
// it doesn't afford us the easy ability to "look ahead," which we need for
// many grammars -- but it suffices for our uses here.

import java.util.Scanner;


public class Parser
{
	private Scanner scanner;
	
	
	// When a Parser is constructed, it is associated with a Scanner that
	// gives it its input.
	public Parser(Scanner scanner)
	{
		this.scanner = scanner;
	}
	

	// parse() parses the input contained in the stream of input wrapped by
	// the Scanner provided when the Parser was constructed.  If the entire
	// stream of input is valid -- i.e., it is in the set of strings described
	// by the grammar -- this method has no other effect.  If the stream of
	// input is not valid, a ParseException is thrown.
	public void parse()
	{
		parseProgram();
		
		if (hasNextWord())
		{
			throw new ParseException();
		}
	}
	
	
	// The methods below implement the individual grammar rules.  There is
	// one method per non-terminal symbol.  The parsing methods make use of
	// a few helper methods that are defined at the bottom of this class.

	
	// Program -> while Question Program end
    //          | hello Exclamation
    //          | alex Program
	private void parseProgram()
	{
		String word = nextWord();
		
		if (word.equals("while"))
		{
			parseQuestion();
			parseProgram();
			expect("end");
		}
		else if (word.equals("hello"))
		{
			parseExclamation();
		}
		else if (word.equals("alex"))
		{
			parseProgram();
		}
		else
		{
			throw new ParseException();
		}
	}

	
	// Question -> how now ?
	//           | when then ?
    private void parseQuestion()
    {
    	String word = nextWord();

		if (word.equals("how"))
		{
			expect("now");
			expect("?");
		}
		else if (word.equals("when"))
		{
			expect("then");
			expect("?");
		}
    	else
    	{
    		throw new ParseException();
    	}
    }

    
    // Exclamation -> i am Reason !
    private void parseExclamation()
    {
    	expect("i");
    	expect("am");
    	parseReason();
    	expect("!");
    }
    
    
    // Reason -> mad | irritated | unhappy | happy
    private void parseReason()
    {
		expectAny("mad", "irritated", "unhappy", "happy");
    }
    

	// expect() reads the next word from this parser's associated Scanner to
	// see if it matches the one that is expected.  If it does, this method
	// has no effect; if it doesn't, the method throws a ParseException.
	//
	// This method should be called whenever the next word in the input must
	// be a particular word.
	private void expect(String wordToExpect)
	{
		if (!wordToExpect.equals(nextWord()))
		{
			throw new ParseException();
		}
	}


	// expectAny() reads the next word from this parser's associated Scanner
	// to see if it matches any of the words that are expected.  If it does,
	// this method has no effect; if it doesn't, the method throws a
	// ParseException.
	private void expectAny(String... wordsToExpect)
	{
		String word = nextWord();

		for (String wordToExpect : wordsToExpect)
		{
			if (wordToExpect.equals(word))
			{
				return;
			}
		}

		throw new ParseException();
	}
	

	// hasNextWord() returns true if there are more words in the input of the
	// associated Scanner, and false otherwise.	
	private boolean hasNextWord()
	{
		return scanner.hasNext();
	}
    

	// nextWord() returns the next word of input from the associated Scanner.
	// If there is no more input available, a ParseException is thrown.
    private String nextWord()
    {
    	if (hasNextWord())
    	{
    		return scanner.next();
    	}
    	else
    	{
    		throw new ParseException();
    	}
    }
}
